from discord import Role
from discord.ext import commands
from discord_slash import SlashContext, cog_ext
from discord_slash.utils.manage_commands import create_option

from jarvis.db.types import Setting


class RolepingCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @cog_ext.cog_subcommand(
        base="roleping",
        name="block",
        description="Add a role to the roleping blocklist",
        options=[
            create_option(
                name="role",
                description="Role to add to blocklist",
                option_type=8,
                required=True,
            )
        ],
    )
    @commands.has_permissions(administrator=True)
    async def _roleping_block(self, ctx: SlashContext, role: Role):
        roles = Setting.get(guild=ctx.guild.id, setting="roleping")
        if not roles:
            roles = Setting(guild=ctx.guild.id, setting="roleping", value=[])

        if role.id in roles.value:
            await ctx.send(
                f"Role `{role.name}` already in blocklist.", hidden=True
            )
            return
        roles.value.append(role.id)
        roles.update()
        await ctx.send(f"Role `{role.name}` added to blocklist.")

    @cog_ext.cog_subcommand(
        base="roleping",
        name="allow",
        description="Remove a role from the roleping blocklist",
        options=[
            create_option(
                name="role",
                description="Role to remove from blocklist",
                option_type=8,
                required=True,
            )
        ],
    )
    @commands.has_permissions(administrator=True)
    async def _roleping_allow(self, ctx: SlashContext, role: Role):
        roles = Setting.get(guild=ctx.guild.id, setting="roleping")
        if not roles:
            await ctx.send("No blocklist configured.", hidden=True)
            return

        if role.id not in roles.value:
            await ctx.send(
                f"Role `{role.name}` not in blocklist.", hidden=True
            )
            return
        roles.value.remove(role.id)
        roles.update()
        await ctx.send(f"Role `{role.name}` removed blocklist.")

    @cog_ext.cog_subcommand(
        base="roleping",
        name="list",
        description="List all blocklisted roles",
    )
    async def _roleping_list(self, ctx: SlashContext):
        roles = Setting.get(guild=ctx.guild.id, setting="roleping")
        if not roles:
            await ctx.send("No blocklist configured.", hidden=True)
            return

        message = "Blocklisted Roles:\n```\n"
        if not roles.value:
            await ctx.send("No roles blocklisted.", hidden=True)
            return
        for role in roles.value:
            role = ctx.guild.get_role(role)
            if not role:
                continue
            message += role.name + "\n"
        message += "```"
        await ctx.send(message)
