import asyncio
from datetime import datetime, timedelta

import discord
from discord.ext import commands
from discord.utils import find

from jarvis.cogs.modlog.utils import get_latest_log, modlog_embed
from jarvis.config import get_config
from jarvis.db.types import Ban, Kick, MongoSort, Mute, Setting
from jarvis.utils import build_embed
from jarvis.utils.field import Field


class ModlogMemberCog(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @commands.Cog.listener()
    async def on_member_ban(self, guild: discord.Guild, user: discord.User):
        modlog = Setting.get(guild=guild.id, setting="modlog")
        if modlog:
            channel = guild.get_channel(modlog.value)
            await asyncio.sleep(0.5)  # Need to wait for audit log
            auditlog = guild.audit_logs(
                limit=50,
                action=discord.AuditLogAction.ban,
                after=datetime.utcnow() - timedelta(seconds=15),
                oldest_first=False,
            )
            log: discord.AuditLogEntry = get_latest_log(auditlog, user)
            admin: discord.User = log.user
            if admin.id == get_config().client_id:
                ban = Ban.get(
                    guild=guild.id,
                    user=user.id,
                    active=True,
                    sort=MongoSort(key="created_at", type="desc"),
                )
                admin = guild.get_member(ban.admin)
            embed = await modlog_embed(
                user,
                admin,
                log,
                "User banned",
                f"{user.mention} was banned from {guild.name}",
            )

            await channel.send(embed=embed)

    @commands.Cog.listener()
    async def on_member_unban(self, guild: discord.Guild, user: discord.User):
        modlog = Setting.get(guild=guild.id, setting="modlog")
        if modlog:
            channel = guild.get_channel(modlog.value)
            await asyncio.sleep(0.5)  # Need to wait for audit log
            auditlog = guild.audit_logs(
                limit=50,
                action=discord.AuditLogAction.unban,
                after=datetime.utcnow() - timedelta(seconds=15),
                oldest_first=False,
            )
            log: discord.AuditLogEntry = get_latest_log(auditlog, user)
            admin: discord.User = log.user
            if admin.id == get_config().client_id:
                ban = Ban.get(
                    guild=guild.id,
                    user=user.id,
                    active=True,
                    sort=MongoSort(key="created_at", type="desc"),
                )
                admin = guild.get_member(ban.admin)
            embed = await modlog_embed(
                user,
                admin,
                log,
                "User unbanned",
                f"{user.mention} was unbanned from {guild.name}",
            )

            await channel.send(embed=embed)

    @commands.Cog.listener()
    async def on_member_remove(self, user: discord.User):
        modlog = Setting.get(guild=user.guild.id, setting="modlog")
        if modlog:
            channel = user.guild.get_channel(modlog.value)
            await asyncio.sleep(0.5)  # Need to wait for audit log
            auditlog = await user.guild.audit_logs(
                limit=50,
                action=discord.AuditLogAction.kick,
                after=datetime.utcnow() - timedelta(seconds=15),
                oldest_first=False,
            )
            log: discord.AuditLogEntry = get_latest_log(auditlog, user)
            admin: discord.User = log.user
            if admin.id == get_config().client_id:
                kick = Kick.get(
                    guild=user.guild.id,
                    user=user.id,
                    sort=MongoSort(key="created_at", type="desc"),
                )
                admin = user.guild.get_member(kick.admin)
            embed = await modlog_embed(
                user,
                admin,
                log,
                "User Kicked",
                f"{user.mention} was kicked from {user.guild.name}",
            )

            await channel.send(embed=embed)

    async def process_mute(self, before, after) -> discord.Embed:
        auditlog = await before.guild.audit_logs(
            limit=50,
            action=discord.AuditLogAction.member_role_update,
            after=datetime.utcnow() - timedelta(seconds=15),
            oldest_first=False,
        ).flatten()
        log: discord.AuditLogEntry = get_latest_log(auditlog, before)
        admin: discord.User = log.user
        if admin.id == get_config().client_id:
            mute = Mute.get(
                guild=before.guild.id,
                user=before.id,
                active=True,
                sort=MongoSort(key="created_at", type="desc"),
            )
            admin = before.guild.get_member(mute.admin)
        return await modlog_embed(
            member=before,
            admin=admin,
            log=log,
            title="User Muted",
            desc=f"{before.mention} was muted",
        )

    async def process_unmute(self, before, after) -> discord.Embed:
        auditlog = await before.guild.audit_logs(
            limit=50,
            action=discord.AuditLogAction.member_role_update,
            after=datetime.utcnow() - timedelta(seconds=15),
            oldest_first=False,
        ).flatten()
        log: discord.AuditLogEntry = get_latest_log(auditlog, before)
        admin: discord.User = log.user
        if admin.id == get_config().client_id:
            mute = Mute.get(
                guild=before.guild.id,
                user=before.id,
                active=True,
                sort=MongoSort(key="created_at", type="desc"),
            )
            mute = Mute(**mute)
            admin = before.guild.get_member(mute.admin)
        return await modlog_embed(
            member=before,
            admin=admin,
            log=log,
            title="User Muted",
            desc=f"{before.mention} was muted",
        )

    async def process_verify(self, before, after) -> discord.Embed:
        auditlog = await before.guild.audit_logs(
            limit=50,
            action=discord.AuditLogAction.member_role_update,
            after=datetime.utcnow() - timedelta(seconds=15),
            oldest_first=False,
        ).flatten()
        log: discord.AuditLogEntry = get_latest_log(auditlog, before)
        admin: discord.User = log.user
        return await modlog_embed(
            member=before,
            admin=admin,
            log=log,
            title="User Verified",
            desc=f"{before.mention} was verified",
        )

    async def process_rolechange(self, before, after) -> discord.Embed:
        auditlog = await before.guild.audit_logs(
            limit=50,
            action=discord.AuditLogAction.member_role_update,
            after=datetime.utcnow() - timedelta(seconds=15),
            oldest_first=False,
        ).flatten()
        log: discord.AuditLogEntry = get_latest_log(auditlog, before)
        admin: discord.User = log.user
        role = None
        title = "User Given Role"
        verb = "was given"
        if len(before.roles) > len(after.roles):
            title = "User Forfeited Role"
            verb = "forfeited"
            role = find(lambda x: x not in after.roles, before.roles)
        elif len(before.roles) < len(after.roles):
            role = find(lambda x: x not in before.roles, after.roles)
        role_text = role.mention if role else "||`[redacted]`||"
        return await modlog_embed(
            member=before,
            admin=admin,
            log=log,
            title=title,
            desc=f"{before.mention} {verb} role {role_text}",
        )

    @commands.Cog.listener()
    async def on_member_update(
        self, before: discord.User, after: discord.User
    ):
        modlog = Setting.get(guild=before.guild.id, setting="modlog")
        if modlog:
            channel = after.guild.get_channel(modlog.value)
            await asyncio.sleep(0.5)  # Need to wait for audit log
            embed = None
            mute = Setting.get(guild=before.guild.id, setting="mute")
            verified = Setting.get(guild=before.guild.id, setting="verified")
            if mute and before.guild.get_role(mute.value) in after.roles:
                embed = await self.process_mute(before, after)
            elif mute and before.guild.get_role(mute.value) in before.roles:
                embed = await self.process_unmute(before, after)
            elif (
                verified
                and before.guild.get_role(verified.value) not in before.roles
                and after.guild.get_role(verified.value) in after.roles
            ):
                embed = await self.process_verify(before, after)
            elif before.nick != after.nick:
                auditlog = await before.guild.audit_logs(
                    limit=50,
                    action=discord.AuditLogAction.member_update,
                    after=datetime.utcnow() - timedelta(seconds=15),
                    oldest_first=False,
                ).flatten()
                log: discord.AuditLogEntry = get_latest_log(auditlog, before)
                bname = before.nick if before.nick else before.name
                aname = after.nick if after.nick else after.name
                fields = [
                    Field(
                        name="Before",
                        value=f"{bname} ({before.name}"
                        + f"#{before.discriminator})",
                    ),
                    Field(
                        name="After",
                        value=f"{aname} ({after.name}"
                        + f"#{after.discriminator})",
                    ),
                ]
                if log.user.id != before.id:
                    fields.append(
                        Field(
                            name="Moderator",
                            value=f"{log.user.mention} ({log.user.name}"
                            + f"#{log.user.discriminator})",
                        )
                    )
                if log.reason:
                    fields.append(
                        Field(name="Reason", value=log.reason, inline=False),
                    )
                embed = build_embed(
                    title="User Nick Changed",
                    description=f"{after.mention} changed their nickname",
                    color="#fc9e3f",
                    fields=fields,
                    timestamp=log.created_at,
                )
                embed.set_author(
                    name=f"{after.name}",
                    icon_url=after.avatar_url,
                )
                embed.set_footer(
                    text=f"{after.name}#{after.discriminator} | {after.id}"
                )
            elif len(before.roles) != len(after.roles):
                # TODO: User got a new role
                embed = await self.process_rolechange(before, after)
            if embed:
                await channel.send(embed=embed)
