import re

from discord import DMChannel, Message
from discord.utils import find

from jarvis import jarvis
from jarvis.config import get_config
from jarvis.db.types import Autopurge, Autoreact, Setting
from jarvis.utils import build_embed
from jarvis.utils.field import Field

invites = re.compile(
    r"(?:https?://)?(?:www.)?(?:discord.(?:gg|io|me|li)|discord(?:app)?.com/invite)/([^\s/]+?)(?=\b)",
    flags=re.IGNORECASE,
)


async def autopurge(message):
    autopurge = Autopurge.get(
        guild=message.guild.id, channel=message.channel.id
    )
    if autopurge:
        await message.delete(delay=autopurge.delay)


async def autoreact(message):
    autoreact = Autoreact.get(
        guild=message.guild.id,
        channel=message.channel.id,
    )
    if autoreact:
        for reaction in autoreact.reactions:
            await message.add_reaction(reaction)


async def checks(message):
    # #tech
    channel = find(
        lambda x: x.id == 599068193339736096, message.channel_mentions
    )
    if channel and message.author.id == 293795462752894976:
        await channel.send(
            content="https://cdn.discordapp.com/attachments/"
            + "664621130044407838/805218508866453554/tech.gif"
        )
    content = re.sub(r"\s+", "", message.content)
    match = invites.search(content)
    if match:
        guild_invites = await message.guild.invites()
        allowed = [x.code for x in guild_invites] + [
            "dbrand",
            "VtgZntXcnZ",
        ]
        if match.group(1) not in allowed:
            await message.delete()
            warning = Warning(
                active=True,
                admin=get_config().client_id,
                duration=24,
                guild=message.guild.id,
                reason="Sent an invite link",
                user=message.author.id,
            )
            warning.insert()
            fields = [
                Field(
                    "Reason",
                    "Sent an invite link",
                    False,
                )
            ]
            embed = build_embed(
                title="Warning",
                description=f"{message.author.mention} has been warned",
                fields=fields,
            )
            embed.set_author(
                name=message.author.nick
                if message.author.nick
                else message.author.name,
                icon_url=message.author.avatar_url,
            )
            embed.set_footer(
                text=f"{message.author.name}#"
                + f"{message.author.discriminator} "
                + f"| {message.author.id}"
            )
            await message.channel.send(embed=embed)


async def massmention(message):
    massmention = Setting.get(
        guild=message.guild.id,
        setting="massmention",
    )
    if (
        massmention.value > 0
        and len(message.mentions)
        - (1 if message.author in message.mentions else 0)
        > massmention.value
    ):
        warning = Warning(
            active=True,
            admin=get_config().client_id,
            duration=24,
            guild=message.guild.id,
            reason="Mass Mention",
            user=message.author.id,
        )
        warning.insert()
        fields = [Field("Reason", "Mass Mention", False)]
        embed = build_embed(
            title="Warning",
            description=f"{message.author.mention} has been warned",
            fields=fields,
        )
        embed.set_author(
            name=message.author.nick
            if message.author.nick
            else message.author.name,
            icon_url=message.author.avatar_url,
        )
        embed.set_footer(
            text=f"{message.author.name}#{message.author.discriminator} "
            + f"| {message.author.id}"
        )
        await message.channel.send(embed=embed)


async def roleping(message):
    roleping = Setting.get(guild=message.guild.id, setting="roleping")
    roles = []
    for mention in message.role_mentions:
        roles.append(mention.id)
    for mention in message.mentions:
        for role in mention.roles:
            roles.append(role.id)
    if (
        roleping
        and any(x in roleping.value for x in roles)
        and not any(x.id in roleping.value for x in message.author.roles)
    ):
        warning = Warning(
            active=True,
            admin=get_config().client_id,
            duration=24,
            guild=message.guild.id,
            reason="Pinged a blocked role/user with a blocked role",
            user=message.author.id,
        )
        warning.insert()
        fields = [
            Field(
                "Reason",
                "Pinged a blocked role/user with a blocked role",
                False,
            )
        ]
        embed = build_embed(
            title="Warning",
            description=f"{message.author.mention} has been warned",
            fields=fields,
        )
        embed.set_author(
            name=message.author.nick
            if message.author.nick
            else message.author.name,
            icon_url=message.author.avatar_url,
        )
        embed.set_footer(
            text=f"{message.author.name}#{message.author.discriminator} "
            + f"| {message.author.id}"
        )
        await message.channel.send(embed=embed)


@jarvis.event
async def on_message(message: Message):
    if (
        not isinstance(message.channel, DMChannel)
        and message.author.id != jarvis.user.id
    ):
        await autoreact(message)
        await massmention(message)
        await roleping(message)
        await autopurge(message)
        await checks(message)
    await jarvis.process_commands(message)
